@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>, 
    except where otherwise indicated.
                                                                      
    This program is free software; you can redistribute it and/or modify
@@ -36,10 +36,13 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+
 
 #include <errno.h>
 
@@ -64,9 +67,9 @@
 #include <ne_socket.h>
 #include <ne_locks.h>
 #include <ne_alloc.h>
+#include <ne_redirect.h>
 
 #include "common.h"
-
 #include "cadaver.h"
 #include "cmdline.h"
 #include "commands.h"
@@ -79,33 +82,23 @@
 static netrc_entry *netrc_list; /* list of netrc entries */
 #endif
 
-ne_session *session;
-ne_lock_store *lock_store = NULL;
 const char *lock_store_fn = NULL;
 
 static char *progname; /* argv[0] */
 
-ne_uri server = {0}; /* current URI of server. */
-
 char *proxy_hostname;
 int proxy_port;
 char *server_username = NULL, *server_password = NULL;
 
+/* Current session state. */
+struct session session;
+
 ne_ssl_client_cert *client_cert;
 
 /* Global options */
 
-char *path, /* current working collection */
-    *old_path; /* previous working collection */
-/* TODO: use dynamically allocated memory for paths, maybe */
-
 int tolerant; /* tolerate DAV-enabledness failure */
 
-int use_expect = 0;
-
-int have_connection = 0; /* true when we are connected to the server */
-int dav_collection;
-
 /* Current output state */
 static enum out_state {
     out_none, /* not doing anything */
@@ -133,8 +126,6 @@ static void usage(void)
 "Usage: %s [OPTIONS] http://hostname[:port]/path\n"
 "  Port defaults to 80, path defaults to '/'\n"
 "Options:\n"
-"  -e, --expect100  Enable sending of `Expect: 100-continue' header.\n"
-"     *** Warning: For Apache servers, use only with version 1.3.9 and later.\n"
 "  -t, --tolerant   Allow cd/open into non-WebDAV enabled collection.\n"
 "  -V, --version    Display version information.\n"
 "  -h, --help       Display this help message.\n"
@@ -144,7 +135,7 @@ static void usage(void)
 static void init_locking(void)
 {
     lock_store_fn = get_option(opt_lockstore);
-    lock_store = ne_lockstore_create();
+    session.locks = ne_lockstore_create();
     /* TODO: read in lock list from ~/.davlocks */
 }
 
@@ -155,11 +146,13 @@ static void finish_locking(void)
 
 void close_connection(void)
 {
-    ne_session_destroy(session);
-    have_connection = false;
-    NE_FREE(path);
-    NE_FREE(old_path);
-    printf(_("Connection to `%s' closed.\n"), server.host);
+    ne_session_destroy(session.sess);
+    session.connected = 0;
+    printf(_("Connection to `%s' closed.\n"),
+           session.uri.host);
+    ne_uri_free(&session.uri);
+    if (session.lastwp)
+        ne_free(session.lastwp);
 }
 
 /* Sets the current collection to the given path.  Returns zero on
@@ -170,16 +163,16 @@ int set_path(const char *newpath)
     int is_coll = (getrestype(newpath) == resr_collection);
     if (is_coll || tolerant) {
 	if (!is_coll) {
-	    dav_collection = false;
+	    session.isdav = 0;
 	    printf(_("Ignored error: %s not WebDAV-enabled:\n%s\n"), newpath,
-		   ne_get_error(session));
+		   ne_get_error(session.sess));
 	} else {
-	    dav_collection = true;
+	    session.isdav = 1;
 	}
 	return 0;
     } else {
 	printf(_("Could not access %s (not WebDAV-enabled?):\n%s\n"), newpath,
-	       ne_get_error(session));
+	       ne_get_error(session.sess));
 	return 1;
     }
 }
@@ -199,7 +192,7 @@ static int cert_verify(void *ud, int failures, const ne_ssl_certificate *c)
 
     if (failures & NE_SSL_IDMISMATCH) {
 	printf(_("Certificate was issued to hostname `%s' rather than `%s'\n"),
-	       ne_ssl_cert_identity(c), server.host);
+	       ne_ssl_cert_identity(c), session.uri.host);
 	printf(_("This connection could have been intercepted.\n"));
     }
 
@@ -259,7 +252,7 @@ static void provide_clicert(void *userdata, ne_session *sess,
     
     if (!ne_ssl_clicert_encrypted(client_cert)) {
         printf("Using client certificate.\n");
-        ne_ssl_set_clicert(session, client_cert);
+        ne_ssl_set_clicert(session.sess, client_cert);
     }
     
 }
@@ -268,14 +261,14 @@ static int setup_ssl(void)
 {
     char *ccfn = get_option(opt_clicert);
 
-    ne_ssl_trust_default_ca(session);
+    ne_ssl_trust_default_ca(session.sess);
 	      
-    ne_ssl_set_verify(session, cert_verify, NULL);
+    ne_ssl_set_verify(session.sess, cert_verify, NULL);
 
     if (ccfn) {
         client_cert = ne_ssl_clicert_read(ccfn);
         if (client_cert) {
-            ne_ssl_provide_clicert(session, provide_clicert, ccfn);
+            ne_ssl_provide_clicert(session.sess, provide_clicert, ccfn);
         } else {
             printf("Could not load client certificate from `%s'.\n",
                    ccfn);
@@ -291,69 +284,72 @@ void open_connection(const char *url)
     char *proxy_host = get_option(opt_proxy), *pnt;
     ne_server_capabilities caps;
     int ret, use_ssl = 0;
+    ne_session *sess;
 
-    if (have_connection) {
+    if (session.connected) {
 	close_connection();
     } else {
-	NE_FREE(path);
-	NE_FREE(old_path);
+        ne_uri_free(&session.uri);
+        if (session.lastwp) {
+            ne_free(session.lastwp);
+            session.lastwp = NULL;
+        }
     }
 
-    server.path = NULL; /* this is used as a temporary store. */
-    ne_uri_free(&server);
-
     /* Single argument: see whether we have a path or scheme */
     if (strchr(url, '/') == NULL) {
 	/* No path, no scheme -> just a hostname */
 	pnt = strchr(url, ':');
 	if (pnt != NULL) {
 	    *pnt++ = '\0';
-	    server.port = atoi(pnt);
+	    session.uri.port = atoi(pnt);
 	} else {
-	    server.port = 80;
+	    session.uri.port = 80;
 	}
-	server.host = ne_strdup(url);
-	server.scheme = ne_strdup("http");
+	session.uri.host = ne_strdup(url);
+	session.uri.scheme = ne_strdup("http");
     } else {
 	/* Parse the URL */
-	if (ne_uri_parse(url, &server) || server.host == NULL) {
+	if (ne_uri_parse(url, &session.uri) || session.uri.host == NULL) {
 	    printf(_("Could not parse URL `%s'\n"), url);
 	    return;
 	}
 
-	if (server.scheme == NULL)
-	    server.scheme = ne_strdup("http");
+	if (session.uri.scheme == NULL)
+	    session.uri.scheme = ne_strdup("http");
 
-	if (!server.port)
-	    server.port = ne_uri_defaultport(server.scheme);
+	if (!session.uri.port)
+	    session.uri.port = ne_uri_defaultport(session.uri.scheme);
 
-	if (strcasecmp(server.scheme, "https") == 0) {
-	    if (!ne_supports_ssl()) {
+	if (strcasecmp(session.uri.scheme, "https") == 0) {
+	    if (!ne_has_support(NE_FEATURE_SSL)) {
 		printf(_("SSL is not enabled.\n"));
 		return;
 	    }
 	    use_ssl = 1;
 	}
     }
-    
-    session = ne_session_create(server.scheme, server.host, server.port);
 
+    session.sess = ne_session_create(session.uri.scheme, session.uri.host,
+                                     session.uri.port);
+    sess = session.sess;
+    
     if (use_ssl && setup_ssl()) {
 	return;
     }
 
-    ne_lockstore_register(lock_store, session);
-    
-    ne_set_progress(session, transfer_progress, NULL);
-    ne_set_status(session, connection_status, NULL);
+    ne_lockstore_register(session.locks, sess);
+    ne_redirect_register(sess);
+    ne_set_progress(sess, transfer_progress, NULL);
+    ne_set_status(sess, connection_status, NULL);
 
-    if (server.path == NULL) {
-	server.path = ne_strdup("/");
+    if (session.uri.path == NULL) {
+	session.uri.path = ne_strdup("/");
     } else {
-	if (!ne_path_has_trailing_slash(server.path)) {
-	    pnt = ne_concat(server.path, "/", NULL);
-	    free(server.path);
-	    server.path = pnt;
+	if (!ne_path_has_trailing_slash(session.uri.path)) {
+	    pnt = ne_concat(session.uri.path, "/", NULL);
+	    free(session.uri.path);
+	    session.uri.path = pnt;
 	}
     }
 
@@ -370,7 +366,7 @@ void open_connection(const char *url)
 #ifdef ENABLE_NETRC
     {
 	netrc_entry *found;
-	found = search_netrc(netrc_list, server.host);
+	found = search_netrc(netrc_list, session.uri.host);
 	if (found != NULL) {
 	    if (found->account && found->password) {
 		server_username = found->account;
@@ -379,45 +375,40 @@ void open_connection(const char *url)
 	}
     }
 #endif /* ENABLE_NETRC */
-    have_connection = false;
+    session.connected = 0;
 
-    ne_set_expect100(session, use_expect);
-    ne_set_useragent(session, "cadaver/" PACKAGE_VERSION);
-    ne_set_server_auth(session, supply_creds_server, NULL);
-    ne_set_proxy_auth(session, supply_creds_proxy, NULL);
+    ne_set_useragent(session.sess, "cadaver/" PACKAGE_VERSION);
+    ne_set_server_auth(session.sess, supply_creds_server, NULL);
+    ne_set_proxy_auth(session.sess, supply_creds_proxy, NULL);
     
     if (proxy_host) {
-	ne_session_proxy(session, proxy_hostname, proxy_port);
+	ne_session_proxy(session.sess, proxy_hostname, proxy_port);
     }
 
-    ret = ne_options(session, server.path, &caps);
+    ret = ne_options(session.sess, session.uri.path, &caps);
     
     switch (ret) {
     case NE_OK:
-	have_connection = true;
-	path = NULL;
-	if (set_path(server.path)) {
+	session.connected = true;
+	if (set_path(session.uri.path)) {
 	    close_connection();
-	} else {
-	    path = server.path;
 	}
 	break;
     case NE_CONNECT:
 	if (proxy_host) {
 	    printf(_("Could not connect to `%s' on port %d:\n%s\n"),
-		   proxy_hostname, proxy_port, ne_get_error(session));
+		   proxy_hostname, proxy_port, ne_get_error(session.sess));
 	} else {
 	    printf(_("Could not connect to `%s' on port %d:\n%s\n"),
-		   server.host, server.port, ne_get_error(session));
+		   session.uri.host, session.uri.port, ne_get_error(session.sess));
 	}
 	break;
     case NE_LOOKUP:
-	puts(ne_get_error(session));
+	puts(ne_get_error(session.sess));
 	break;
     default:
-	/* FIXME: This is NOT a "could not open connection" error */
-	printf(_("Could not contact server:\n%s\n"),
-	       ne_get_error(session));
+	printf(_("Could not open collection:\n%s\n"),
+	       ne_get_error(session.sess));
 	break;
     }
 
@@ -441,7 +432,6 @@ static void parse_args(int argc, char **argv)
     static const struct option opts[] = {
 	{ "version", no_argument, NULL, 'V' },
 	{ "help", no_argument, NULL, 'h' },
-	{ "expect100", no_argument, NULL, 'e' },
 	{ "proxy", required_argument, NULL, 'p' },
 	{ "tolerant", no_argument, NULL, 't' },
 	{ 0, 0, 0, 0 }
@@ -451,7 +441,6 @@ static void parse_args(int argc, char **argv)
 	switch (optc) {
 	case 'h': usage(); exit(-1);
 	case 'V': execute_about(); exit(-1);
-	case 'e': use_expect = true; break;
 	case 'p': set_proxy(optarg); break;
 	case 't': tolerant = 1; break;
 	case '?': 
@@ -476,17 +465,19 @@ static void parse_args(int argc, char **argv)
     }
 }
 
-static char * read_command(void)
+static char *read_command(void)
 {
     static char prompt[BUFSIZ];
-    if (path) {
-        char *p = ne_path_unescape(path);
-	snprintf(prompt, BUFSIZ, "dav:%s%c ", p,
-		  dav_collection?'>':'?');
+
+    if (session.uri.path) {
+        char *p = ne_path_unescape(session.uri.path);
+	ne_snprintf(prompt, BUFSIZ, "dav:%s%c ", p,
+                    session.isdav ? '>' : '?');
         ne_free(p);
     } else {
 	sprintf(prompt, "dav:!> ");
     }
+
     return readline(prompt); 
 }
 
@@ -494,7 +485,7 @@ static int execute_command(const char *line)
 {
     const struct command *cmd;
     char **tokens;
-    int argcount, ret = 0;
+    int n, argcount, ret = 0;
     tokens = parse_command(line, &argcount);
     if (argcount == 0) {
 	free(tokens);
@@ -524,7 +515,7 @@ static int execute_command(const char *line)
 	} else {
 	    printf(".\n");
 	}
-    } else if (!have_connection && cmd->needs_connection) {
+    } else if (!session.connected && cmd->needs_connection) {
 	printf(_("The `%s' command can only be used when connected to the server.\n"
 		  "Try running `open' first (see `help open' for more details).\n"), 
 		  tokens[0]);
@@ -553,7 +544,10 @@ static int execute_command(const char *line)
 	    break;
 	}
     }
-    split_string_free(tokens);
+    for (n = 0; n < argcount; n++) {
+        ne_free(tokens[n]);
+    }
+    ne_free(tokens);
     return ret;
 }
 
@@ -568,7 +562,7 @@ static RETSIGTYPE quit_handler(int sig)
 	return;
     } else {
 	printf(_("Terminated by signal %d.\n"), sig);
-	if (have_connection) {
+	if (session.connected) {
 	    close_connection();
 	}
 	exit(-1);
@@ -650,7 +644,7 @@ static char *remote_completion(const char *text, int state)
 	 */
 	if (last_fetch < (time(NULL) - COMPLETION_CACHE_EXPIRE) 
 	    || !last_path 
-	    || strcmp(path, last_path) != 0) {
+	    || strcmp(session.uri.path, last_path) != 0) {
 
 	    if (last_path != NULL) {
 		free(last_path);
@@ -661,14 +655,15 @@ static char *remote_completion(const char *text, int state)
 	    }
 
 	    /* Hide the connection status */
-	    ne_set_status(session, NULL, NULL);
-	    if (fetch_resource_list(session, path, 1, 0, &reslist) != NE_OK) {
+	    ne_set_status(session.sess, NULL, NULL);
+	    if (fetch_resource_list(session.sess, session.uri.path, 1, 0, 
+                                    &reslist) != NE_OK) {
 		reslist = NULL;
 	    }
 	    /* Restore the session connection printing */
-	    ne_set_status(session, connection_status, NULL);
+	    ne_set_status(session.sess, connection_status, NULL);
 
-	    last_path = ne_strdup(path);
+	    last_path = ne_strdup(session.uri.path);
 	}
 
 	current = reslist;
@@ -730,7 +725,7 @@ static char **completion(const char *text, int start, int end)
 		/* TODO */
 		break;
 	    case parmscope_remote:
-		if (have_connection) {
+		if (session.connected) {
 		    matches = rl_completion_matches(text, remote_completion);
 		}
 		break;
@@ -837,10 +832,12 @@ int main(int argc, char *argv[])
     int ret = 0;
     char *home = getenv("HOME"), *tmp;
 
-    path = NULL;
-    have_connection = false;
     progname = argv[0];
 
+#ifdef HAVE_SETLOCALE
+    setlocale(LC_ALL, "");
+#endif
+
 #ifdef ENABLE_NLS
     bindtextdomain(PACKAGE_NAME, LOCALEDIR);
     textdomain(PACKAGE_NAME);
@@ -855,6 +852,8 @@ int main(int argc, char *argv[])
 
     ne_sock_init();
 
+    memset(&session, 0, sizeof session);
+
     /* Options before rcfile, so rcfile settings can
      * override defaults */
     tmp = ne_concat(home, "/.cadaver-locks", NULL);
@@ -887,7 +886,7 @@ int main(int argc, char *argv[])
 	}
     }
 
-    if (have_connection) {
+    if (session.connected) {
 	close_connection();
     }
 
@@ -1088,7 +1087,6 @@ static int supply_creds(const char *prompt, const char *realm, const char *hostn
 	return -1;
     } else if (strlen(tmp) >= NE_ABUFSIZ) {
 	printf(_("\rPassword too long (>%d)\n"), NE_ABUFSIZ);
-	memset(tmp, strlen(tmp), 0);
 	return -1;
     }
     
@@ -1125,7 +1123,7 @@ static int supply_creds_server(void *userdata, const char *realm, int attempt,
 
     return supply_creds(
 	_("Authentication required for %s on server `%s':\n"), realm,
-	server.host, username, password);
+	session.uri.host, username, password);
 }
 
 static int supply_creds_proxy(void *userdata, const char *realm, int attempt,
