@@ -1,6 +1,6 @@
 /* 
    cadaver, command-line DAV client
-   Copyright (C) 1999-2002, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>, 
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -93,7 +93,7 @@ static int is_lockable(const char *uri)
     /* TODO: for the mo, just check we're on a Class 2 server.
      * Should check supportedlock property really. */
 
-    if (ne_options(session, uri, &caps) != NE_OK) {
+    if (ne_options(session.sess, uri, &caps) != NE_OK) {
 	return 0;
     }
 
@@ -119,7 +119,7 @@ void execute_edit(const char *remote)
     int fd;
     int is_checkout, is_checkin;
     
-    real_remote = resolve_path(path, remote, false);
+    real_remote = resolve_path(session.uri.path, remote, false);
 
     /* Don't let them edit a collection, since PUT to a collection is
      * bogus. Really we want to be able to fetch a "DefaultDocument"
@@ -150,7 +150,11 @@ void execute_edit(const char *remote)
     }
 
     /* Sanity check on the file perms. */
+#ifdef HAVE_FCHMOD
     if (fchmod(fd, 0600) == -1) {
+#else
+    if (chmod(fname, 0600) == -1) {
+#endif
 	printf(_("Could not set file permissions for %s:\n%s\n"), fname,
 	       strerror(errno));
 	goto edit_bail;
@@ -158,12 +162,12 @@ void execute_edit(const char *remote)
    
     if (can_lock) {
 	lock = ne_lock_create();
-	ne_fill_server_uri(session, &lock->uri);
+	ne_fill_server_uri(session.sess, &lock->uri);
 	lock->uri.path = ne_strdup(real_remote);
 	lock->owner = getowner();
 	out_start("Locking", remote);
-	if (out_handle(ne_lock(session, lock))) {
-	    ne_lockstore_add(lock_store, lock);
+	if (out_handle(ne_lock(session.sess, lock))) {
+	    ne_lockstore_add(session.locks, lock);
 	} else {
 	    ne_lock_destroy(lock);
 	    goto edit_close;
@@ -182,7 +186,7 @@ void execute_edit(const char *remote)
     output(o_transfer, _("Downloading `%s' to %s"), real_remote, fname);
 
     /* Don't puke if get fails -- perhaps we are creating a new one? */
-    out_result(ne_get(session, real_remote, fd));
+    out_result(ne_get(session.sess, real_remote, fd));
     
     if (close(fd)) {
 	output(o_finish, _("Error writing to temporary file: %s\n"), 
@@ -202,7 +206,7 @@ void execute_edit(const char *remote)
 		       real_remote);
 		/* FIXME: conditional PUT using fetched Etag/modtime if
 		 * !can_lock */
-		if (out_handle(ne_put(session, real_remote, fd))) {
+		if (out_handle(ne_put(session.sess, real_remote, fd))) {
 		    upload_okay = 1;
 		} else {
 		    /* TODO: offer to save locally instead */
@@ -230,8 +234,8 @@ void execute_edit(const char *remote)
     /* UNLOCK it again whether we succeed or failed in our mission */
     if (can_lock) {
 	output(o_start, "Unlocking `%s':", remote);
-	out_result(ne_unlock(session, lock));
-	ne_lockstore_remove(lock_store, lock);
+	out_result(ne_unlock(session.sess, lock));
+	ne_lockstore_remove(session.locks, lock);
 	ne_lock_destroy(lock);
     }
 
