@@ -1,6 +1,6 @@
 /* 
    'ls' for cadaver
-   Copyright (C) 2000-2004, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 2000-2004, 2006, Joe Orton <joe@manyfish.co.uk>, 
    except where otherwise indicated.
                                                                      
    This program is free software; you can redistribute it and/or modify
@@ -121,7 +121,7 @@ static void display_ls_line(struct resource *res)
 	exec_char = res->is_executable ? '*' : ' ';
 	/* 0: no vcr, 1: checkin, 2: checkout */
 	vcr_char = res->is_vcr==0 ? ' ' : (res->is_vcr==1? '>' : '<');
-	printf("%5s %c%c%-29s %10d  %s\n", 
+	printf("%5s %c%c%-29s %10" NE_FMT_TIME_T "  %s\n", 
 	       restype, vcr_char, exec_char, name,
 	       res->size, format_time(res->modtime));
     }
@@ -134,13 +134,14 @@ void execute_ls(const char *remote)
     int ret;
     char *real_remote;
     struct resource *reslist = NULL, *current, *next;
+
     if (remote != NULL) {
-	real_remote = resolve_path(path, remote, true);
+	real_remote = resolve_path(session.uri.path, remote, true);
     } else {
-	real_remote = ne_strdup(path);
+	real_remote = ne_strdup(session.uri.path);
     }
     out_start(_("Listing collection"), real_remote);
-    ret = fetch_resource_list(session, real_remote, 1, 0, &reslist);
+    ret = fetch_resource_list(session.sess, real_remote, 1, 0, &reslist);
     if (ret == NE_OK) {
 	/* Easy this, eh? */
 	if (reslist == NULL) {
@@ -161,7 +162,12 @@ void execute_ls(const char *remote)
     free(real_remote);
 }
 
-static void results(void *userdata, const char *uri,
+static void results(void *userdata, 
+#if NE_VERSION_MINOR < 26
+                    const char *path,
+#else
+                    const ne_uri *uri,
+#endif
 		    const ne_prop_result_set *set)
 {
     struct fetch_context *ctx = userdata;
@@ -169,32 +175,37 @@ static void results(void *userdata, const char *uri,
     const char *clength, *modtime, *isexec;
     const char *checkin, *checkout;
     const ne_status *status = NULL;
+
+#if NE_VERSION_MINOR < 26
     ne_uri u;
-
-    NE_DEBUG(NE_DBG_HTTP, "Uri: %s\n", uri);
-
-    newres = ne_propset_private(set);
-    
-    if (ne_uri_parse(uri, &u))
-	return;
-    
-    if (u.path == NULL) {
-	ne_uri_free(&u);
-	return;
+            
+    if (ne_uri_parse(path, &u)) {
+        return;
     }
 
-    NE_DEBUG(NE_DBG_HTTP, "URI path %s in %s\n", u.path, ctx->target);
-    
-    if (ne_path_compare(ctx->target, u.path) == 0 && !ctx->include_target) {
+    if (u.path == NULL) {
+        ne_uri_free(&u);
+        return;
+    }
+    path = u.path;
+#else
+    const char *path = uri->path;
+#endif
+
+    newres = ne_propset_private(set);
+
+    if (ne_path_compare(ctx->target, path) == 0 && !ctx->include_target) {
 	/* This is the target URI */
 	NE_DEBUG(NE_DBG_HTTP, "Skipping target resource.\n");
 	/* Free the private structure. */
-	free(newres);
-	ne_uri_free(&u);
+	ne_free(newres);
+#if NE_VERSION_MINOR < 26
+        ne_uri_free(&u);
+#endif
 	return;
     }
 
-    newres->uri = ne_strdup(u.path);
+    newres->uri = ne_strdup(path);
 
     clength = ne_propset_value(set, &ls_props[0]);    
     modtime = ne_propset_value(set, &ls_props[1]);
@@ -267,7 +278,9 @@ static void results(void *userdata, const char *uri,
     }
     newres->next = current;
 
+#if NE_VERSION_MINOR < 26
     ne_uri_free(&u);
+#endif
 }
 
 static int ls_startelm(void *userdata, int parent, 
@@ -293,8 +306,8 @@ static int ls_startelm(void *userdata, int parent,
 
 void free_resource(struct resource *res)
 {
-    NE_FREE(res->uri);
-    NE_FREE(res->error_reason);
+    if (res->uri) ne_free(res->uri);
+    if (res->error_reason) ne_free(res->error_reason);
     free(res);
 }
 
@@ -307,7 +320,13 @@ void free_resource_list(struct resource *res)
     }
 }
 
-static void *create_private(void *userdata, const char *uri)
+static void *create_private(void *userdata, 
+#if NE_VERSION_MINOR < 26
+                    const char *path
+#else
+                    const ne_uri *uri
+#endif
+    )
 {
     return ne_calloc(sizeof(struct resource));
 }
@@ -328,7 +347,11 @@ int fetch_resource_list(ne_session *sess, const char *uri,
     ne_xml_push_handler(ne_propfind_get_parser(pfh), 
                         ls_startelm, NULL, NULL, pfh);
 
-    ne_propfind_set_private(pfh, create_private, NULL);
+    ne_propfind_set_private(pfh, create_private, NULL
+#if NE_VERSION_MINOR > 25
+                            ,NULL
+#endif
+        );
 
     ret = ne_propfind_named(pfh, ls_props, results, &ctx);
 
