@@ -1,6 +1,6 @@
 /* 
    'search' for cadaver
-   Copyright (C) 2004, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2004-2006, Joe Orton <joe@manyfish.co.uk>
    Copyright (C) 2002, GRASE Lab, UCSC <grase@cse.ucsc.edu>, 
    except where otherwise indicated.
                                                                      
@@ -69,6 +69,9 @@
 
 #define WORDLEN 256		/*Max length of a identifier(token) in the search command */
 
+#undef EOL
+#define EOL "\n"
+
 /* Dead prop */
 typedef struct dead_prop
 {
@@ -210,12 +213,18 @@ static int comparison_value(char **string_parsed, ne_buffer * result_buf);
 static int word_string(char **string_parsed, ne_buffer * result_buf);
 
 /* Set xml parser error */
-static void set_xml_error(search_ctx * sctx, const char *format, ...)
+static void set_xml_error(search_ctx *sctx, const char *format, ...)
 {
-    sctx->err_code = NE_ERROR;
-    ne_set_error(session, format);
-}
+    va_list ap;
+    char buf[512];
 
+    va_start(ap, format);
+    ne_vsnprintf(buf, sizeof buf, format, ap);
+    va_end(ap);
+
+    ne_set_error(session.sess, "%s", buf);
+    sctx->err_code = NE_ERROR;
+}
 
 static int start_element(void *userdata, int parent,
 			 const char *nspace, 
@@ -466,33 +475,33 @@ static void search_ctx_destroy(search_ctx * sctx)
     ne_buffer_destroy(sctx->cdata);
 
     for (res = sctx->root; res;) {
-	NE_FREE(res->href);
+	if (res->href) ne_free(res->href);
 	/* live props */
-	NE_FREE(res->creationdate);
-	NE_FREE(res->displayname);
-	NE_FREE(res->getcontentlanguage);
-	NE_FREE(res->getcontentlength);
-	NE_FREE(res->getcontenttype);
-	NE_FREE(res->getetag);
-	NE_FREE(res->getlastmodified);
-	NE_FREE(res->lockdiscovery);
-	NE_FREE(res->resourcetype);
-	NE_FREE(res->source);
-	NE_FREE(res->supportedlock);
-	NE_FREE(res->collection);
+	if (res->creationdate) ne_free(res->creationdate);
+	if (res->displayname) ne_free(res->displayname);
+	if (res->getcontentlanguage) ne_free(res->getcontentlanguage);
+	if (res->getcontentlength) ne_free(res->getcontentlength);
+	if (res->getcontenttype) ne_free(res->getcontenttype);
+	if (res->getetag) ne_free(res->getetag);
+	if (res->getlastmodified) ne_free(res->getlastmodified);
+	if (res->lockdiscovery) ne_free(res->lockdiscovery);
+	if (res->resourcetype) ne_free(res->resourcetype);
+	if (res->source) ne_free(res->source);
+	if (res->supportedlock) ne_free(res->supportedlock);
+	if (res->collection) ne_free(res->collection);
 
 	for (dprop = res->root; dprop;) {
-	    NE_FREE(dprop->nspace);
-	    NE_FREE(dprop->name);
-	    NE_FREE(dprop->value);
+	    if (dprop->nspace) ne_free(dprop->nspace);
+	    if (dprop->name) ne_free(dprop->name);
+	    if (dprop->value) ne_free(dprop->value);
 
 	    dprop_free = dprop;
 	    dprop = dprop->next;
-	    NE_FREE(dprop_free);
+	    ne_free(dprop_free);
 	}
 	res_free = res;
 	res = res->next;
-	NE_FREE(res_free);
+	ne_free(res_free);
     }
 }
 
@@ -525,7 +534,7 @@ static ne_propname *order_props_create(const char *str)
     props = ne_calloc(sizeof(ne_propname) * (num_props + 1));
 
     /* Set first token */
-    NE_FREE(buf);
+    ne_free(buf);
     buf = ne_strdup(str);
     tok = strtok(buf, delm);
 
@@ -538,7 +547,7 @@ static ne_propname *order_props_create(const char *str)
 	props[n].name = ne_strdup(tok);
     }
 
-    NE_FREE(buf);
+    ne_free(buf);
     return props;
 }
 
@@ -658,7 +667,7 @@ void execute_search(int count, const char **args)
     printf("%s, ", query->data);
 
     /* Run search and get data to sctx */
-    ret = run_search(session, path, searchdepth, query, sctx);
+    ret = run_search(session.sess, session.uri.path, searchdepth, query, sctx);
     if (ret == NE_OK) {
 	display_results(sctx);
     }
@@ -676,7 +685,7 @@ static int search_select_gen(const ne_propname * props,
     int n;
 
     if (!basic_search) {
-	ne_set_error(session, "select_gen: no buffer");
+	ne_set_error(session.sess, "select_gen: no buffer");
 	return NE_ERROR;
     }
 
@@ -704,7 +713,7 @@ static int search_from_gen(const char *href, const int depth,
     const char *depth_str;
 
     if (!basic_search || !href) {
-	ne_set_error(session, "from_gen: no buffer or no href");
+	ne_set_error(session.sess, "from_gen: no buffer or no href");
 	return NE_ERROR;
     }
 
@@ -746,7 +755,7 @@ static int search_where_gen(const char *condition_str,
     ne_buffer *result_buf;
 
     if (!basic_search || !condition_str) {
-	ne_set_error(session, "where_gen: no buffer or no query");
+	ne_set_error(session.sess, "where_gen: no buffer or no query");
 	return NE_ERROR;
     }
 
@@ -756,15 +765,15 @@ static int search_where_gen(const char *condition_str,
     ne_buffer_zappend(basic_search, "<D:where>" EOL);
 
     if (search_condition(&string_parsed, result_buf) == NE_ERROR) {
-	NE_FREE(ptr_backup);
+	ne_free(ptr_backup);
 	ne_buffer_destroy(result_buf);
 	return NE_ERROR;	/*Parsing error */
     }
 
     /*The ending of a condition must be an ENDBUF */
     if (read_aword(&string_parsed, identifier) != ENDBUF) {
-	ne_set_error(session, "Syntax error in the search condition.");
-	NE_FREE(ptr_backup);
+	ne_set_error(session.sess, "Syntax error in the search condition.");
+	ne_free(ptr_backup);
 	ne_buffer_destroy(result_buf);
 	return NE_ERROR;
     }
@@ -774,7 +783,7 @@ static int search_where_gen(const char *condition_str,
 
     ne_buffer_zappend(basic_search, "</D:where>" EOL);
 
-    NE_FREE(ptr_backup);
+    ne_free(ptr_backup);
     ne_buffer_destroy(result_buf);
 
     return NE_OK;
@@ -787,7 +796,7 @@ static int search_orderby_gen(const ne_propname * asc,
     int n;
 
     if (!basic_search) {
-	ne_set_error(session, "orderby_gen: no buffer or no query");
+	ne_set_error(session.sess, "orderby_gen: no buffer or no query");
 	return NE_ERROR;
     }
 
@@ -993,7 +1002,7 @@ static int first_word_equal(const char *string_parsed,
 
     read_aword(&string_buffer, first_word);
 
-    NE_FREE(ptr_backup);
+    ne_free(ptr_backup);
 
     if (strcasecmp(first_word, word_to_compare) == 0)
 	return 1;		/*equal */
@@ -1024,7 +1033,7 @@ static int first_word_integer(const char *string_parsed)
     else
 	ret = 0;
 
-    NE_FREE(ptr_backup);
+    ne_free(ptr_backup);
 
     return ret;
 }				/*End of first_word_compare */
@@ -1230,7 +1239,7 @@ static int boolean_primary(char **string_parsed, ne_buffer * result_buf)
 
 	/*Read and match ")" */
 	if (match_fetch(string_parsed, ")") == NE_ERROR) {
-	    ne_set_error(session,
+	    ne_set_error(session.sess,
 			 "Syntax error: A ')' is expected in the search condition.");
 	    ne_buffer_destroy(sub_result);
 	    return NE_ERROR;	/*parsing error */
@@ -1327,7 +1336,7 @@ static int predicate(char **string_parsed, ne_buffer * result_buf)
 
     /*Read the column name */
     if (read_aword(string_parsed, column_name) != IDEN) {
-	ne_set_error(session,
+	ne_set_error(session.sess,
 		     "A column name is expected in the search condition.");
 	ne_buffer_destroy(comparing_value);
 	return NE_ERROR;	/*Parsing error */
@@ -1338,7 +1347,7 @@ static int predicate(char **string_parsed, ne_buffer * result_buf)
 
     /*Translate the operator to XML form */
     if (operator_translate(operator, XML_operator) == 0) {
-	ne_set_error(session,
+	ne_set_error(session.sess,
 		     "Syntax error: Invalid operator in the search condition.");
 	ne_buffer_destroy(comparing_value);
 	return NE_ERROR;
@@ -1356,7 +1365,7 @@ static int predicate(char **string_parsed, ne_buffer * result_buf)
 	else {
 	    /* For the case of word string. */
 	    if (word_string(string_parsed, comparing_value) == NE_ERROR) {
-		ne_set_error(session,
+		ne_set_error(session.sess,
 			     "Syntax error: A quoted string or a word string is expected in the search condition.");
 		ne_buffer_destroy(comparing_value);
 		return NE_ERROR;	/*Parsing error */
@@ -1409,7 +1418,7 @@ int contains_predicate(char **string_parsed, ne_buffer * result_buf)
     /*Read 'contains' */
     if (match_fetch(string_parsed, "contains") == NE_ERROR) {
 	/*The case of <contains predicate> */
-	ne_set_error(session,
+	ne_set_error(session.sess,
 		     "Syntax error: A 'contains' is expected in the search condition.");
 	ne_buffer_destroy(contain_string);
 	return NE_ERROR;
@@ -1426,7 +1435,7 @@ int contains_predicate(char **string_parsed, ne_buffer * result_buf)
     else {
 	/* For the case of <wordstring> */
 	if (word_string(string_parsed, contain_string) == NE_ERROR) {
-	    ne_set_error(session,
+	    ne_set_error(session.sess,
 			 "Syntax error: A quoted string or a word string is expected in the search condition.");
 	    ne_buffer_destroy(contain_string);
 	    return NE_ERROR;	/*Parsing error */
@@ -1462,7 +1471,7 @@ static int quoted_string(char **string_parsed, ne_buffer * result_buf)
 
     /*Read a quotation mark */
     if (match_fetch(string_parsed, "'") == NE_ERROR) {
-	ne_set_error(session,
+	ne_set_error(session.sess,
 		     "Syntax error: A ' is expected in the search condition.");
 	return NE_ERROR;	/*Parsing error */
     }
@@ -1480,7 +1489,7 @@ static int quoted_string(char **string_parsed, ne_buffer * result_buf)
     }
 
     if (current_char != '\'') {	/*There should be a ending ' */
-	ne_set_error(session,
+	ne_set_error(session.sess,
 		     "An ending ' is expected in the search condition.");
 	return NE_ERROR;	/*parsing error */
     }
@@ -1572,7 +1581,7 @@ static int comparison_value(char **string_parsed, ne_buffer * result_buf)
 	}
 	else {			/* It is the case of a word string */
 	    if (word_string(string_parsed, comparing_value) == NE_ERROR) {
-		ne_set_error(session,
+		ne_set_error(session.sess,
 			     "Syntax error: An integer, quoted string or word string is expected in the search condition.");
 		ne_buffer_destroy(comparing_value);
 		return NE_ERROR;
